/*
 * Copyright 2010-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 * 
 *  http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

/*
 * Do not modify this file. This file is generated from the autoscaling-2011-01-01.normal.json service model.
 */
using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using System.Text;
using System.IO;

using Amazon.Runtime;
using Amazon.Runtime.Internal;

namespace Amazon.AutoScaling.Model
{
    /// <summary>
    /// Container for the parameters to the CreateOrUpdateTags operation.
    /// Creates or updates tags for the specified Auto Scaling group.
    /// 
    ///  
    /// <para>
    /// A tag is defined by its resource ID, resource type, key, value, and propagate flag.
    /// The value and the propagate flag are optional parameters. The only supported resource
    /// type is <code>auto-scaling-group</code>, and the resource ID must be the name of the
    /// group. The <code>PropagateAtLaunch</code> flag determines whether the tag is added
    /// to instances launched in the group. Valid values are <code>true</code> or <code>false</code>.
    /// </para>
    ///  
    /// <para>
    /// When you specify a tag with a key that already exists, the operation overwrites the
    /// previous tag definition, and you do not get an error message.
    /// </para>
    ///  
    /// <para>
    /// For more information, see <a href="http://docs.aws.amazon.com/AutoScaling/latest/DeveloperGuide/ASTagging.html">Tagging
    /// Auto Scaling Groups and Instances</a> in the <i>Auto Scaling Developer Guide</i>.
    /// </para>
    /// </summary>
    public partial class CreateOrUpdateTagsRequest : AmazonAutoScalingRequest
    {
        private List<Tag> _tags = new List<Tag>();

        /// <summary>
        /// Gets and sets the property Tags. 
        /// <para>
        /// One or more tags.
        /// </para>
        /// </summary>
        public List<Tag> Tags
        {
            get { return this._tags; }
            set { this._tags = value; }
        }

        // Check to see if Tags property is set
        internal bool IsSetTags()
        {
            return this._tags != null && this._tags.Count > 0; 
        }

    }
}