/*
 * Copyright 2010-2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 * 
 *  http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

/*
 * Do not modify this file. This file is generated from the logs-2014-03-28.normal.json service model.
 */
using System;
using System.Collections.Generic;
using System.Xml.Serialization;
using System.Text;
using System.IO;

using Amazon.Runtime;
using Amazon.Runtime.Internal;

namespace Amazon.CloudWatchLogs.Model
{
    /// <summary>
    /// Container for the parameters to the PutMetricFilter operation.
    /// Creates or updates a metric filter and associates it with the specified log group.
    /// Metric filters allow you to configure rules to extract metric data from log events
    /// ingested through <code class="code">PutLogEvents</code> requests. 
    /// 
    ///  
    /// <para>
    ///  The maximum number of metric filters that can be associated with a log group is 100.
    /// 
    /// </para>
    /// </summary>
    public partial class PutMetricFilterRequest : AmazonCloudWatchLogsRequest
    {
        private string _filterName;
        private string _filterPattern;
        private string _logGroupName;
        private List<MetricTransformation> _metricTransformations = new List<MetricTransformation>();

        /// <summary>
        /// Empty constructor used to set  properties independently even when a simple constructor is available
        /// </summary>
        public PutMetricFilterRequest() { }

        /// <summary>
        /// Instantiates PutMetricFilterRequest with the parameterized properties
        /// </summary>
        /// <param name="logGroupName">The name of the log group to associate the metric filter with.</param>
        /// <param name="filterName">A name for the metric filter.</param>
        /// <param name="filterPattern">A valid CloudWatch Logs filter pattern for extracting metric data out of ingested log events.</param>
        /// <param name="metricTransformations">A collection of information needed to define how metric data gets emitted.</param>
        public PutMetricFilterRequest(string logGroupName, string filterName, string filterPattern, List<MetricTransformation> metricTransformations)
        {
            _logGroupName = logGroupName;
            _filterName = filterName;
            _filterPattern = filterPattern;
            _metricTransformations = metricTransformations;
        }

        /// <summary>
        /// Gets and sets the property FilterName. 
        /// <para>
        /// A name for the metric filter.
        /// </para>
        /// </summary>
        public string FilterName
        {
            get { return this._filterName; }
            set { this._filterName = value; }
        }

        // Check to see if FilterName property is set
        internal bool IsSetFilterName()
        {
            return this._filterName != null;
        }

        /// <summary>
        /// Gets and sets the property FilterPattern. 
        /// <para>
        /// A valid CloudWatch Logs filter pattern for extracting metric data out of ingested
        /// log events.
        /// </para>
        /// </summary>
        public string FilterPattern
        {
            get { return this._filterPattern; }
            set { this._filterPattern = value; }
        }

        // Check to see if FilterPattern property is set
        internal bool IsSetFilterPattern()
        {
            return this._filterPattern != null;
        }

        /// <summary>
        /// Gets and sets the property LogGroupName. 
        /// <para>
        /// The name of the log group to associate the metric filter with.
        /// </para>
        /// </summary>
        public string LogGroupName
        {
            get { return this._logGroupName; }
            set { this._logGroupName = value; }
        }

        // Check to see if LogGroupName property is set
        internal bool IsSetLogGroupName()
        {
            return this._logGroupName != null;
        }

        /// <summary>
        /// Gets and sets the property MetricTransformations. 
        /// <para>
        /// A collection of information needed to define how metric data gets emitted.
        /// </para>
        /// </summary>
        public List<MetricTransformation> MetricTransformations
        {
            get { return this._metricTransformations; }
            set { this._metricTransformations = value; }
        }

        // Check to see if MetricTransformations property is set
        internal bool IsSetMetricTransformations()
        {
            return this._metricTransformations != null && this._metricTransformations.Count > 0; 
        }

    }
}